from sqlalchemy import event, inspect
from sqlalchemy.orm import object_mapper, ColumnProperty

from wedeliver_core import WeDeliverCore, Auth
from wedeliver_core.helpers.kafka_producers.log_model_changes import log_model_changes


def init_log_model():
    app = WeDeliverCore.get_app()
    db = app.extensions['sqlalchemy'].db

    class LogModel(db.Model):
        __abstract__ = True
        __log_column_changes__ = []

    def get_old_value(attribute_state):
        history = attribute_state.history
        return history.deleted[0] if history.deleted else None

    def trigger_attribute_change_events(object_, is_deleted=False, is_added=False):
        column_changes = None
        for mapper_property in object_mapper(object_).iterate_properties:
            if isinstance(mapper_property, ColumnProperty):
                key = mapper_property.key
                watched_columns = dict()
                for wc in object_.__log_column_changes__:
                    column, alias = wc.split(' as ') if ' as ' in wc else [wc, wc]
                    watched_columns[column] = alias

                if key not in watched_columns.keys():
                    continue

                attribute_state = inspect(object_).attrs.get(key)
                history = attribute_state.history

                if history.has_changes():
                    value = attribute_state.value
                    # old_value is None for new objects and old value for dirty objects
                    old_value = get_old_value(attribute_state)
                    if not column_changes:
                        column_changes = []
                    column_changes.append(dict(
                        column=watched_columns[key],
                        from_value=old_value,
                        to_value=value
                    ))
        if is_added or is_deleted or column_changes:
            action_type = 'updated'
            if is_deleted:
                action_type = 'deleted'
            elif is_added:
                action_type = 'added'
            changes = dict(
                created_by=Auth.get_user_str(),
                service_name=app.config.get("SERVICE_NAME"),
                model_name=object_.__tablename__,
                model_instance_id=object_.id,
                column_changes=column_changes,
                action_type=action_type
            )
            log_changes(changes)

    def log_changes(changes):
        log_model_changes(changes)

    def on_after_flush(session, flush_context):
        try:
            deleted_objects = session.deleted
            changed_objects = session.dirty
            new_objects = session.new
            for o in changed_objects:
                if session.is_modified(o, include_collections=True) and hasattr(o,
                                                                                '__log_column_changes__') and o.__log_column_changes__:
                    trigger_attribute_change_events(o)
            for o in deleted_objects:
                if hasattr(o, '__log_column_changes__') and o.__log_column_changes__:
                    trigger_attribute_change_events(o, is_deleted=True)
            for o in new_objects:
                if hasattr(o, '__log_column_changes__') and o.__log_column_changes__:
                    trigger_attribute_change_events(o, is_added=True)
        except Exception as e:
            app.logger.error("error while logging changes: ", str(e))

    event.listen(db.session, "after_flush", on_after_flush)

    return LogModel



import re
from wedeliver_core.helpers.get_country_code import get_country_code


def validate_mobile_number(mobile):
    mobile = mobile.replace("+", "").replace(" ", "").lstrip("0")
    if len(mobile) < 12:
        country_code = get_country_code()
        if country_code == "sa":
            dial = "966"
        elif country_code == "ps":
            dial = "970"
        elif country_code == "eg":
            dial = "20"
        else:
            return False
        mobile = "{0}{1}".format(dial, int(mobile))
    if (
        re.match(
            r"^(009665|9665|\+9665|05|5)(5|0|3|6|4|9|1|8|7)([0-9]{7})$", str(mobile)
        )
        or re.match(r"^(009705|9705|\+9705|05|5)([0-9]{8})$", str(mobile))
        or re.match(r"^(201|01|\+201|00201)[0-9]{9}$", str(mobile))
    ):
        return mobile
    else:
        return False



from wedeliver_core import WeDeliverCore
from wedeliver_core.helpers.auth import Auth


def init_base_model():
    app = WeDeliverCore.get_app()
    db = app.extensions['sqlalchemy'].db

    class BaseModel(db.Model):
        __abstract__ = True
        id = db.Column(db.Integer, primary_key=True)
        creation = db.Column(db.DateTime, default=db.func.now())
        created_by = db.Column(db.String(64), default=Auth.get_user_str)

        modification = db.Column(db.DateTime, default=db.func.now(), onupdate=db.func.now())

        # def __init__(self, created_by=None):
        #     self.created_by = created_by or Auth.get_user().get("email", "Guest")

    return BaseModel



# -*- coding: utf-8 -*-

import os
import json

from dotenv import load_dotenv


class Config(object):
    SQLALCHEMY_DATABASE_URI = None

    def init_database_config(self):
        self.SQLALCHEMY_DATABASE_URI = "{engine}://{username}:{password}@{server}:{port}/{database}?charset=utf8".format(
            engine=os.environ.get("DATABASE_ENGINE"),
            username=os.environ.get("DATABASE_USERNAME"),
            password=os.environ.get("DATABASE_PASSWORD"),
            server=os.environ.get("DATABASE_SERVER"),
            database=os.environ.get("DATABASE_NAME"),
            port=os.environ.get("DATABASE_PORT", 3306),
        )

    def __init__(self):
        if not os.environ.get("DATABASE_ENGINE"):
            load_dotenv()

        self.init_database_config()

        for k, v in os.environ.items():

            if not k:
                continue

            env_val = os.environ.get(k).strip()
            # Convert Null and empty value to None
            env_val = None if env_val.upper() in ["NULL", ""] else env_val
            # Convert True and False value to Boolean
            env_val = (
                json.loads(env_val.lower())
                if env_val and env_val.upper() in ["TRUE", "FALSE"]
                else env_val
            )

            self.__dict__[k] = env_val
